"""Repeatedly ask QUEST to add you into a particular course."""

# Expose metadata.
from .version import (
    __author__,
    __version__,
    __credits__,
)

def addcourse(user, password, classlist):
    """Repeatedly query QUEST to add classes from classlist until one of
    them works.

    """

    from .course_adder import addcourse as _addcourse
    return _addcourse(user, password, classlist)

def numbers(course):
    """Query uwaterloo.ca for a list of class numbers that correspond to
    the lecture sections of course.

    """

    from .class_numbers import numbers as _numbers
    return _numbers(course)



# Main method.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


"""The main method for this package."""

import argparse
import getpass
try:
    import readline
except ImportError:
    pass
from .course_adder import addcourse
from . import __version__


def main():
    """Main function."""

    descr = """Repeatedly ask QUEST to add you into a particular course."""
    epi = """Report Bugs to the bug list on our github page at:
    <https://github.com/kcolford/uwaterloo-addcourse/issues>"""

    parser = argparse.ArgumentParser(
        prog='addcourse',
        description=descr,
        epilog=epi,
        fromfile_prefix_chars='@',
    )
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + __version__)
    parser.add_argument('-c', '--course', action='store',
                        help='the course to try getting in to')
    parser.add_argument('-u', '--userid', action='store',
                        help='the userid to login as')

    args = parser.parse_args()

    course = args['course']
    if not course:
        course = raw_input('Desired Course: ')
    user = args['userid']
    if not user:
        user = raw_input('QUEST ID: ')
    password = getpass.getpass('Password: ')
    addcourse(user, password, course)



# Interact with the QUEST course page.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


"""Class that interacts with the QUEST course add page."""


import re
from .browser import QuestBrowser


class AddPage(QuestBrowser):
    """Interact with the course add page."""

    def get_add(self):
        """Go to the enrollment page."""

        # Select the enroll page.
        self.do_action('DERIVED_SSS_SCR_SSS_LINK_ANCHOR3')

        # Select the add tab.
        add_link = self.page.find(text='add').parent
        self.make_request(add_link['href'])
        self.get_page()

        # Select the upcomming term if the option is given.
        t = self.page.find(text=term_regex)
        if t and t.parent.name == 'span':
            self.do_action('DERIVED_SSS_SCT_SSR_PB_GO',
                           {'SSR_DUMMY_RECV1$sels$0': "1"})

    def setup_post(self):
        """Setup a POST request from QUEST."""

        for n in self.page.find_all(name='input', attrs={'type': 'hidden'}):
            self.add_form(n['name'], n['value'])

    def do_action(self, action, d={}):
        """Do a POST action on the add class page."""

        self.make_request(enroll_add_page)
        self.setup_post()
        self.add_form('ICAction', action)
        self.add_forms(d)
        self.get_page()


enroll_add_page = '/psc/AS/ACADEMIC/SA/c/SA_LEARNER_SERVICES.SSR_SSENRL_CART.GBL'
term_regex = re.compile(r'^(Winter|Spring|Fall) [[:digit:]]{4}$')



# Module for looking at class numbers.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


import urllib
import urllib2
import re
import time
import bs4

def nearest_term(t=time.time()):
    """Return the number for the nearest term."""

    t = time.localtime(t)
    y = t.tm_year % 100
    m = int((t.tm_mon + 2) / 4) * 4 + 1
    return 1000 + y * 10 + m


def numbers(course):
    """Return a list of class numbers corresponding to course."""

    subject, number = re.findall(r'[a-zA-Z]+|[0-9]+', course)
    query = {'level': 'under',
             'sess': nearest_term(),
             'subject': subject.upper(),
             'cournum': number}

    f = urllib2.urlopen(course_query_url, urllib.urlencode(query))
    a = bs4.BeautifulSoup(f.read())
    f.close()

    return [int(n.parent.previous)
            for n in a.findAll(text=re.compile('LEC'))]


course_query_url = 'http://www.adm.uwaterloo.ca/cgi-bin/cgiwrap/infocour/salook.pl'
